import { ɵɵdefineInjectable, Injectable, EventEmitter, Component, ViewChild, Input, Output, ɵɵinject, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { DialogModule } from 'primeng/dialog';
import { ButtonModule } from 'primeng/button';
import { JnyPreHttpServicesService } from 'jny-pre-http-services';
import { Subject } from 'rxjs';

class JnyUserPickerService {
    constructor() {
        this.dialogAlertSubject = new Subject();
    }
    // ===== loginModal (START)=====
    showDialogAlert(_header, _content) {
        this.sendDialogAlertSubject({
            display: true,
            header: _header,
            content: _content
        });
    }
    sendDialogAlertSubject(obj) {
        this.dialogAlertSubject.next(obj);
    }
    getDialogAlertSubject() {
        return this.dialogAlertSubject.asObservable();
    }
}
JnyUserPickerService.ɵprov = ɵɵdefineInjectable({ factory: function JnyUserPickerService_Factory() { return new JnyUserPickerService(); }, token: JnyUserPickerService, providedIn: "root" });
JnyUserPickerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
JnyUserPickerService.ctorParameters = () => [];

class JnyUserPickerComponent {
    constructor(httpService, jnyUserPickerService) {
        this.httpService = httpService;
        this.jnyUserPickerService = jnyUserPickerService;
        this.ctrlUrl = 'osjcEmpCtrl/';
        this.selectEmpDept = '---';
        this.selectEmpName = '---';
        this.valueChange = new EventEmitter();
        this.userInfo = new EventEmitter();
        this.STORAGE_KEY = 'JNY_loginUser';
        this.isLoading = false;
        this.selectEmp = {
            deptNo: '',
            userNo: '',
            cname: ''
        };
    }
    ngOnChanges(changes) {
        // 決定不處理 selectEmpName 和 selectEmpDept
        // 只要傳入工號 value 就觸發查詢
        for (let propertyName in changes) {
            if (propertyName === "value") {
                let change = changes[propertyName];
                // 也不管之前的值為何, 就丟去changeUser做檢查並查詢
                // let cur  = JSON.stringify(change.currentValue);
                // let prev = JSON.stringify(change.previousValue);
                this.changeUser();
            }
        }
    }
    ngOnInit() {
    }
    changeUser() {
        // 避免valueEmit之後又再觸發changeUser事件
        if (this.selectEmp.userNo === this.value) {
            return false;
        }
        if (this.value.length !== 6) {
            this.selectEmpDept = '---';
            this.selectEmpName = '---';
            this.userInfo.emit({ empNo: '', empName: '', empDept: '' });
        }
        else {
            this.selectEmp.userNo = this.value;
            this.checkAuth();
        }
    }
    qryEmp(deptNo) {
        return this.httpService.post(this.ctrlUrl + 'qryEmp', deptNo);
    }
    // 挑選部門
    pickDept() {
        this.qryEmp(this.selectDept).subscribe((carrier) => {
            this.setEmpVos(carrier);
            this.addAuth();
        }, error => console.log(error));
    }
    pickUser(event) {
        this.checkAuth();
    }
    // 查詢目前登入的人是否有此使用者權限
    checkAuth() {
        const params = { userId: this.selectEmp.userNo };
        //this.httpService.post('oajcContactCtrl/queryEmpNoForPicker', params).subscribe(
        this.isLoading = true;
        this.httpService.post(this.ctrlUrl + 'queryEmpByUserId', params).subscribe((data) => {
            this.setValueEmit(data);
            this.isLoading = false;
            this.setuidTextFocus();
        }, error => {
            console.log(error);
            this.isLoading = false;
            this.setuidTextFocus();
        });
    }
    setValueEmit(data) {
        if (data.attributeMap.userInfo.empNo === '') {
            this.value = '';
            this.selectEmpDept = '---';
            this.selectEmpName = '---';
        }
        else {
            this.value = data.attributeMap.userInfo.empNo;
            this.selectEmpDept = data.attributeMap.userInfo.deptNo;
            this.selectEmpName = data.attributeMap.userInfo.empName;
        }
        this.visible = false;
        this.valueChange.emit(this.value);
        this.userInfo.emit(data.attributeMap.userInfo);
    }
    // 開啟使用者挑選清單
    showUserPicker() {
        this.queryDeptVOs();
        this.getUserByDept();
        //  this.getUserInfo();
    }
    // 依照部門代碼取得使用者清單
    getUserByDept() {
        let loginUserVO = this.getLoginUser();
        if (this.value != null && this.value !== '') {
            // 使用者輸入欄已有值時，預設將使用者選單選到該選項,部門選單則不選到任何值
            this.selectDept = '';
            this.qryEmpBySameDept(this.value);
        }
        else {
            // 使用者輸入欄無值時，選單預設選到為登入者的部門和職鯿
            this.selectDept = loginUserVO.deptNo;
            this.qryEmpBySameDept(loginUserVO.userId);
        }
        this.visible = true;
    }
    qryEmpBySameDept(empNo) {
        this.httpService.post(this.ctrlUrl + 'qryEmpBySameDept', empNo).subscribe((carrier) => {
            this.setEmpVos(carrier);
            this.selectEmp.userNo = empNo;
        }, error => { console.error(error); });
    }
    setEmpVos(carrier) {
        let mapArray = carrier.attributeMap['empList'];
        this.empVOs = new Array();
        for (let map of mapArray) {
            let empVO = new EmpVO();
            empVO.userNo = map['USERNO'];
            empVO.cname = map['CNAME'];
            empVO.deptNo = map['DEPTNO'];
            empVO.hasAuth = '0';
            this.empVOs.push(empVO);
        }
    }
    // empVOs 添加 auth 權限
    addAuth() {
        let auth_list = JSON.parse(sessionStorage.getItem('auth-list'));
        if (auth_list == null) {
            return;
        }
        this.empVOs.forEach(function (mval) {
            auth_list.forEach(function (aval) {
                if (mval.userNo === aval.empNo) {
                    mval.hasAuth = aval.hasAuth;
                }
            });
        });
    }
    queryDeptVOs() {
        this.httpService.post(this.ctrlUrl + 'qryAllDept').subscribe((carrier) => {
            let mapArray = carrier.attributeMap['deptList'];
            this.deptVOs = new Array();
            for (let map of mapArray) {
                let deptVO = new DeptVO();
                deptVO.deptNo = map['DEPTNO'];
                deptVO.deptName = map['DEPTNAME'];
                this.deptVOs.push(deptVO);
            }
        }, error => console.log(error));
    }
    getLoginUser() {
        const loginUserVO = JSON.parse(sessionStorage.getItem(this.STORAGE_KEY));
        if (loginUserVO == null || loginUserVO === undefined) {
            return new LoginUserVO();
        }
        else {
            return loginUserVO;
        }
    }
    setuidTextFocus() {
        setTimeout(() => {
            this.uidTextElement.nativeElement.focus();
        }, 0);
    }
}
JnyUserPickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-jny-user-picker',
                template: "<style>\r\n  .myBox {\r\n    display: inline-block;\r\n  }\r\n</style>\r\n\r\n<div class=\"p-inputgroup\" style='width:180px'>\r\n  <button type=\"button\" pButton pRipple icon=\"pi pi-user\" styleClass=\"p-button-info\"\r\n    (click)=\"showUserPicker()\"></button>\r\n  <input type=\"text\" pInputText [(ngModel)]=\"value\" (input)=\"changeUser()\" maxlength=\"6\"\r\n    class='p-inputtext p-component' [disabled]=\"isLoading\" #uidText>\r\n  <span class=\"p-inputgroup-addon\"><span class=\"badge badge-info mr-1\">{{selectEmpDept}}</span>{{selectEmpName}}</span>\r\n</div>\r\n\r\n<p-dialog header=\"\u4EBA\u54E1\u9078\u55AE\" [(visible)]=\"visible\" [modal]=\"true\" [responsive]=\"true\"\r\n  [style]=\"{width: '450px', minWidth: '400px'}\" [baseZIndex]=\"9999\">\r\n  <div style=\"text-align: center\">\r\n    <div class=\"myBox\">\r\n      <!--\u56E0IE\u4E0B\u62C9\u9078\u55AEoption\u6C92\u6709click\u4E8B\u4EF6\uFF0C\u6545\u5BEB\u5728select\u4E2D-->\r\n      <select [(ngModel)]=\"selectDept\" size=\"20\" (click)=\"pickDept()\" style=\"width:200px\">\r\n        <option *ngFor=\"let deptVO of deptVOs\" [value]=\"deptVO.deptNo\">{{deptVO.deptNo + ' '+ deptVO.deptName}}</option>\r\n      </select>\r\n    </div>\r\n    <div class=\"myBox\" style=\"margin-left:10px;\">\r\n      <select [(ngModel)]=\"selectEmp\" size=\"20\" (click)=\"pickUser($event)\" id=\"pickUserSelect\" style=\"width:160px\">\r\n        <option *ngFor=\"let empVO of empVOs\" [ngValue]=\"empVO\">\r\n          <!--[disabled]=\"empVO.hasAuth!=='1'\"-->\r\n          <!--[value]=\"empVO.userNo\"-->\r\n          {{empVO.cname}}\r\n        </option>\r\n      </select>\r\n    </div>\r\n  </div>\r\n</p-dialog>\r\n",
                providers: []
            },] }
];
JnyUserPickerComponent.ctorParameters = () => [
    { type: JnyPreHttpServicesService },
    { type: JnyUserPickerService }
];
JnyUserPickerComponent.propDecorators = {
    uidTextElement: [{ type: ViewChild, args: ['uidText',] }],
    value: [{ type: Input }],
    selectEmpDept: [{ type: Input }],
    selectEmpName: [{ type: Input }],
    valueChange: [{ type: Output }],
    userInfo: [{ type: Output }]
};
class DeptVO {
}
class EmpVO {
}
class LoginUserVO {
    constructor() {
        this.tokenId = '';
        this.userId = '';
        this.userName = '';
        this.postId = '';
        this.postName = '';
        this.deptNo = '';
        this.deptName = '';
    }
}

class JnyDeptPickerService {
    constructor(httpService) {
        this.httpService = httpService;
    }
    //取得部門資料
    getDeptList() {
        let url = 'osjcEmpCtrl/qryAllDept';
        return this.httpService.post(url);
    }
}
JnyDeptPickerService.ɵprov = ɵɵdefineInjectable({ factory: function JnyDeptPickerService_Factory() { return new JnyDeptPickerService(ɵɵinject(JnyPreHttpServicesService)); }, token: JnyDeptPickerService, providedIn: "root" });
JnyDeptPickerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
JnyDeptPickerService.ctorParameters = () => [
    { type: JnyPreHttpServicesService }
];

class JnyDeptPickerComponent {
    constructor(jnyDeptPickerService) {
        this.jnyDeptPickerService = jnyDeptPickerService;
        this.deptVOs = [];
        this.valueChange = new EventEmitter();
        this.selectDept = {
            DEPTNO: "",
            DEPTNAME: ""
        };
    }
    ngOnChanges(changes) {
        for (let propertyName in changes) {
            if (propertyName === "value") {
                this.changeDept();
            }
        }
    }
    ngOnInit() {
        this.getDeptList();
    }
    changeDept() {
        // 元件首次載入的ngOnChanges尚未取得所有單位資料
        if (this.deptVOs.length === 0) {
            return false;
        }
        // 避免valueEmit之後又再觸發changeUser事件
        if (this.selectDept.DEPTNO === this.value) {
            return false;
        }
        this.selectDept.DEPTNO = (this.value) ? this.value.trim().toUpperCase() : '';
        this.selectDept.DEPTNAME = '---';
        let app = this;
        if (this.selectDept.DEPTNO.length > 0) {
            let fullDeptNo = app.selectDept.DEPTNO;
            let deptNoLength = app.selectDept.DEPTNO.length;
            if (deptNoLength == 1)
                fullDeptNo += "000";
            else if (deptNoLength == 2)
                fullDeptNo += "00";
            else if (deptNoLength == 3)
                fullDeptNo += "0";
            this.deptVOs.forEach(function (deptVo) {
                if (deptVo.DEPTNO.startsWith(fullDeptNo)) {
                    app.selectDept.DEPTNO = deptVo.DEPTNO;
                    app.selectDept.DEPTNAME = deptVo.DEPTNAME;
                    return false;
                }
            });
        }
    }
    getDeptList() {
        this.jnyDeptPickerService.getDeptList().subscribe((data) => {
            this.deptVOs = data.attributeMap.deptList;
            // 首次執行的ngOnChanges優先於getDeptList
            // 因此在getDeptList完成時強制執行changeDept
            this.changeDept();
        }, error => console.log(error));
        ;
    }
    pickDept() {
        this.value = this.selectDept.DEPTNO;
        this.visible = false;
        this.valueChange.emit(this.value);
    }
    showDeptPicker() {
        this.visible = true;
    }
}
JnyDeptPickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-jny-dept-picker',
                template: "<div class=\"p-inputgroup\" style='width:160px'>\r\n  <button type=\"button\" pButton pRipple icon=\"pi pi-users\" styleClass=\"p-button-info\"\r\n    (click)=\"showDeptPicker()\"></button>\r\n  <input type=\"text\" pInputText [(ngModel)]=\"value\" (input)=\"changeDept()\" maxlength=\"4\"\r\n    class='p-inputtext p-component'>\r\n  <!--\r\n    <span class=\"p-inputgroup-addon\"><span class=\"badge badge-info mr-1\">{{selectDept.DEPTNO}}</span>{{selectDept.DEPTNAME}}</span>\r\n  -->\r\n  <span class=\"p-inputgroup-addon\">{{selectDept.DEPTNAME}}</span>\r\n</div>\r\n<p-dialog header=\"\u90E8\u9580\u9078\u55AE\" [(visible)]=\"visible\" [modal]=\"true\" [responsive]=\"true\"\r\n  [style]=\"{width: '400px', minWidth: '400px'}\" [baseZIndex]=\"9999\">\r\n  <div style=\"text-align: center\">\r\n    <!--\u56E0IE\u4E0B\u62C9\u9078\u55AEoption\u6C92\u6709click\u4E8B\u4EF6\uFF0C\u6545\u5BEB\u5728select\u4E2D-->\r\n    <select [(ngModel)]=\"selectDept\" size=\"15\" (click)=\"pickDept()\" style=\"width:100%\">\r\n      <option *ngFor=\"let deptVO of deptVOs\" [ngValue]=\"deptVO\">{{deptVO.DEPTNO + ' '+ deptVO.DEPTNAME}}</option>\r\n    </select>\r\n  </div>\r\n</p-dialog>\r\n",
                providers: [JnyDeptPickerService],
                styles: [""]
            },] }
];
JnyDeptPickerComponent.ctorParameters = () => [
    { type: JnyDeptPickerService }
];
JnyDeptPickerComponent.propDecorators = {
    value: [{ type: Input }],
    selectDeptName: [{ type: Input }],
    valueChange: [{ type: Output }]
};

class JnyDialogAlertComponent {
    constructor(jnyUserPickerService) {
        this.jnyUserPickerService = jnyUserPickerService;
        this.display = false;
        this.subscription = this.jnyUserPickerService.getDialogAlertSubject().subscribe((obj) => {
            this.display = obj['display'];
            this.header = obj['header'];
            this.content = obj['content'];
        });
    }
    ngOnInit() {
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
    }
}
JnyDialogAlertComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-jny-dialog-alert',
                template: "<p-dialog [header]=\"header\" [(visible)]=\"display\" [modal]=\"true\" [draggable]=\"false\" [resizable]=\"false\" position=\"top\"\r\n  [style]=\"{width: '50vw'}\">\r\n  {{content}}\r\n  <p-footer>\r\n    <p-button icon=\"pi pi-check\" (click)=\"display=false\" label=\"Ok\" styleClass=\"p-button-text\"></p-button>\r\n  </p-footer>\r\n</p-dialog>\r\n",
                styles: [""]
            },] }
];
JnyDialogAlertComponent.ctorParameters = () => [
    { type: JnyUserPickerService }
];

class JnyUserPickerModule {
}
JnyUserPickerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [JnyUserPickerComponent, JnyDialogAlertComponent, JnyDeptPickerComponent],
                imports: [
                    FormsModule,
                    CommonModule,
                    DialogModule,
                    ButtonModule
                ],
                exports: [JnyUserPickerComponent, JnyDeptPickerComponent, JnyDialogAlertComponent]
            },] }
];

/*
 * Public API Surface of jny-user-picker
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DeptVO, EmpVO, JnyDeptPickerComponent, JnyDeptPickerService, JnyDialogAlertComponent, JnyUserPickerComponent, JnyUserPickerModule, JnyUserPickerService, LoginUserVO };
//# sourceMappingURL=jny-user-picker.js.map
